# ManualGeometryBase
Base classes: Device
## <a name='serializedGeometry'></a>Serialized geometry (`serializedGeometry`)
Type
: STRING

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: empty string

## Geometry preview (`geometryPreview`)
### Pixels (`geometryPreview.pixels`)
The N-dimensional array containing the pixels
#### <a name='geometryPreview.pixels.data'></a>Data (`geometryPreview.pixels.data`)
Type
: BYTE_ARRAY

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The data of the array as an untyped buffer of bytes

#### <a name='geometryPreview.pixels.shape'></a>Shape (`geometryPreview.pixels.shape`)
Type
: VECTOR_UINT64

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The shape of the array reflects total dimensionality and each element the extension in its dimension (0: any extension)

Default value
: ``[0]``

#### <a name='geometryPreview.pixels.type'></a>Data Type (`geometryPreview.pixels.type`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The type of the contained array data

Default value
: ``14``

#### <a name='geometryPreview.pixels.isBigEndian'></a>Is big-endian (`geometryPreview.pixels.isBigEndian`)
Type
: BOOL

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: A boolean flag which is true if the data is big-endian

Default value
: ``False``

### <a name='geometryPreview.pixels'></a>Pixels (`geometryPreview.pixels`)
Type
: NDARRAY

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The N-dimensional array containing the pixels

### <a name='geometryPreview.dims'></a>Dimensions (`geometryPreview.dims`)
Type
: VECTOR_UINT64

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The length of the array reflects total dimensionality and each element the extension in this dimension

Default value
: ``[0]``

### <a name='geometryPreview.dimTypes'></a>Dimension Types (`geometryPreview.dimTypes`)
Type
: VECTOR_INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Any dimension should have an enumerated type

Default value
: ``[]``

### <a name='geometryPreview.dimScales'></a>Dimension Scales (`geometryPreview.dimScales`)
Type
: STRING

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: 

Default value
: empty string

### <a name='geometryPreview.encoding'></a>Encoding (`geometryPreview.encoding`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Describes the color space of pixel encoding of the data(e.g. GRAY, RGB, JPG, PNG etc.

Default value
: ``-1``

### <a name='geometryPreview.bitsPerPixel'></a>Bits per pixel (`geometryPreview.bitsPerPixel`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The number of bits needed for each pixel

Default value
: ``8``

### <a name='geometryPreview.roiOffsets'></a>ROI Offsets (`geometryPreview.roiOffsets`)
Type
: VECTOR_UINT64

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Describes the offset of the Region-of-Interest. It willcontain zeros if the image has no ROI defined

Default value
: ``[0]``

### <a name='geometryPreview.binning'></a>Binning (`geometryPreview.binning`)
Type
: VECTOR_UINT64

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The number of binned adjacent pixels. They are reported out of the camera as a single pixel.

Default value
: ``[1]``

### <a name='geometryPreview.rotation'></a>Rotation (`geometryPreview.rotation`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: The image counterclockwise rotation.

Options:
: ``0``, ``90``, ``180``, ``270``

Default value
: ``0``

### <a name='geometryPreview.flipX'></a>Flip X (`geometryPreview.flipX`)
Type
: BOOL

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Image horizontal flip.

Default value
: ``False``

### <a name='geometryPreview.flipY'></a>Flip Y (`geometryPreview.flipY`)
Type
: BOOL

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Image vertical flip.

Default value
: ``False``

## <a name='geometryPreview'></a>Geometry preview (`geometryPreview`)
Type
: HASH

Access mode
: READONLY

Assignment
: OPTIONAL

## Geometry file (`geometryFile`)
Allows loading geometry from CrystFEL geometry file
### <a name='geometryFile.filePath'></a>File path (`geometryFile.filePath`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Full path (including filename and suffix) to the desired geometry file. Keep in mind that the default directory is $KARABO/var/data on device server node, so it's probably wise to give absolute path.

Default value
: empty string

### <a name='geometryFile.fileType'></a>File type (`geometryFile.fileType`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: What kind of file will be loaded. Corresponds to options within EXtra-geom. Note that the options listed here may not all apply to all geometries. I think 'crystfel' (uses extra_geom.[geometry type].from_crystfel_geom is the most 'universal', so I left that as default.

Options:
: ``crystfel``, ``h5``, ``h5+quadrants``

Default value
: ``crystfel``

### Offset (`geometryFile.offset`)
See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.
#### <a name='geometryFile.offset.x'></a>`geometryFile.offset.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

#### <a name='geometryFile.offset.y'></a>`geometryFile.offset.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='geometryFile.offset'></a>Offset (`geometryFile.offset`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.

### <a name='geometryFile.updateManualOnLoad'></a>Update manual settings (`geometryFile.updateManualOnLoad`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: If this flag is on, the manual settings on this device will be updated according to the loaded geometry file. This is useful when you want to load a file and then tweak the geometry a bit. This will zero current offset.

Default value
: ``True``

### <a name='geometryFile.loadFromFile'></a>Load from file (`geometryFile.loadFromFile`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='geometryFile'></a>Geometry file (`geometryFile`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Allows loading geometry from CrystFEL geometry file

## Tweak geometry (`tweakGeometry`)
### <a name='tweakGeometry.undo'></a>Undo (`tweakGeometry.undo`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### <a name='tweakGeometry.redo'></a>Redo (`tweakGeometry.redo`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### Offset (`tweakGeometry.offset`)
See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.
#### <a name='tweakGeometry.offset.x'></a>`tweakGeometry.offset.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

#### <a name='tweakGeometry.offset.y'></a>`tweakGeometry.offset.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='tweakGeometry.offset'></a>Offset (`tweakGeometry.offset`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.

### <a name='tweakGeometry.add'></a>Add offset (`tweakGeometry.add`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### <a name='tweakGeometry.undoLength'></a>Undo length (`tweakGeometry.undoLength`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0``

### <a name='tweakGeometry.redoLength'></a>Redo length (`tweakGeometry.redoLength`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0``

## <a name='tweakGeometry'></a>Tweak geometry (`tweakGeometry`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='updatePreview'></a>Update preview (`updatePreview`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL
