import operator

from karabo.middlelayer import AccessMode, Assignment, String
from .. import base_condition
from ..corrections.JungfrauCorrection import GainModes, GainSettings


def gain_mode_translator(gain_mode_string):
    if gain_mode_string in {"dynamic", "forceswitchg1", "forceswitchg2"}:
        return GainModes.ADAPTIVE_GAIN.name
    elif gain_mode_string in {"fixg1", "fixg2"}:
        return GainModes.FIXED_GAIN.name
    else:
        raise ValueError(f"Unknown gain mode {gain_mode_string}")


def gain_setting_translator(setting):
    if setting == "gain0":
        return GainSettings.LOW_CDS.name
    elif setting == "highgain0":
        return GainSettings.HIGH_CDS.name
    else:
        raise ValueError(f"Unknown gain setting {setting}")


class JungfrauCondition(base_condition.ConditionBase):
    controlDeviceId = String(
        displayedName="Control device ID",
        assignment=Assignment.MANDATORY,
        accessMode=AccessMode.INITONLY,
    )

    @property
    def keys_to_get(self):
        return {
            self.controlDeviceId.value: [
                # cells: 1.0 or 16.0
                ("storageCells", "memoryCells", lambda n: n + 1),
                (
                    "exposureTime",
                    "integrationTime",
                    lambda n: n * 1e6,
                ),
                ("highVoltage", "biasVoltage", operator.itemgetter(0)),
                ("settings", "gainSetting", gain_setting_translator),
                ("gainMode", "gainMode", gain_mode_translator),
            ]
        }
