import numpy as np

from karabo.bound import NODE_ELEMENT, NDARRAY_ELEMENT, DOUBLE_ELEMENT

from .base_addon import BaseCorrectionAddon


class LitPixelCounter(BaseCorrectionAddon):
    def __init__(self, config):
        global cupy
        import cupy

        self._threshold = config["threshold"]

    def reconfigure(self, changed_config):
        if changed_config.has("threshold"):
            self._threshold = changed_config["threshold"]

    @staticmethod
    def extend_output_schema(schema):
        (
            NODE_ELEMENT(schema)
            .key("litpixels")
            .commit(),

            NDARRAY_ELEMENT(schema)
            .key("litpixels.count")
            .dtype('UINT32')
            .commit(),

            NDARRAY_ELEMENT(schema)
            .key("litpixels.unmasked")
            .dtype('FLOAT')
            .commit()
        )

    @staticmethod
    def extend_device_schema(schema, prefix):
        (
            DOUBLE_ELEMENT(schema)
            .key(f"{prefix}.threshold")
            .tags("managed")
            .assignmentOptional()
            .defaultValue(6.0)
            .reconfigurable()
            .commit()
        )

    def post_correction(self, processed_data, cell_table, pulse_table, output_hash):
        n_cells, n_x, n_y = processed_data.shape
        per_asic_data = processed_data.reshape(n_cells, 64, n_x // 64, 64, n_y // 64)

        lit_pixels = np.sum(per_asic_data > self._threshold, axis=(1, 3))
        unmasked_pixels = np.isfinite(per_asic_data).sum(axis=(1, 3))

        output_hash["litpixels.count"] = lit_pixels.get()
        output_hash["litpixels.unmasked"] = unmasked_pixels.get()
