import numpy as np
from karabo.bound import DOUBLE_ELEMENT, NDARRAY_ELEMENT, NODE_ELEMENT, Unit
from .base_addon import BaseCorrectionAddon


class RandomFrames(BaseCorrectionAddon):
    @staticmethod
    def extend_device_schema(schema, prefix):
        (
            DOUBLE_ELEMENT(schema)
            .key(f"{prefix}.probability")
            .tags("managed")
            .unit(Unit.PERCENT)
            .assignmentOptional()
            .defaultValue(50)
            .reconfigurable()
            .commit(),
        )

    @staticmethod
    def extend_output_schema(schema):
        (
            NODE_ELEMENT(schema)
            .key("data")
            .commit(),  # TODO: check if rest of node needs rebuilding if exists

            NDARRAY_ELEMENT(schema)
            .key("data.dataFramePattern")
            .dtype("UINT8")
            .commit(),
        )

    def __init__(self, config):
        # TODO: figure out why no / 100 here...
        self._probability = config["probability"]

    def post_correction(self, processed_data, cell_table, pulse_table, output_hash):
        output_hash["data.dataFramePattern"] = (
            np.random.random(cell_table.size) < self._probability
        ).astype(np.uint8)

    def reconfigure(self, changed_config):
        if changed_config.has("probability"):
            self._probability = changed_config["probability"] / 100
