from karabo.bound import Hash


from calng.DetectorAssembler import DetectorAssembler


class NotADetectorAssembler(DetectorAssembler):
    def __init__(self):
        self._parameters = Hash()

    def get(self, key):
        return self._parameters.get(key)

    def set(self, key, value):
        return self._parameters.set(key, value)


def test_agipd_mapping():
    d = NotADetectorAssembler()
    spb_agipd_mapping = {
        s: i
        for i, s in enumerate(
            [
                "SPB_DET_AGIPD1M-1/CAL/CORRECT00_Q1M1:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT01_Q1M2:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT02_Q1M3:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT03_Q1M4:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT04_Q2M1:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT05_Q2M2:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT06_Q2M3:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT07_Q2M4:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT08_Q3M1:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT09_Q3M2:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT10_Q3M3:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT11_Q3M4:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT12_Q4M1:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT13_Q4M2:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT14_Q4M3:preview.outputRaw",
                "SPB_DET_AGIPD1M-1/CAL/CORRECT15_Q4M4:preview.outputRaw",
            ]
        )
    }
    subset = list(spb_agipd_mapping.keys())
    del subset[4:7]
    d.set("sources", [Hash("select", True, "source", s) for s in subset])
    d.set("sourceToModuleIndex", [])
    d._merge_source_to_index_from_regex()
    d._set_source_to_index_from_table()
    for source in subset:
        assert spb_agipd_mapping[source] == d._source_to_index[source]
    assert len(subset) == len(d._source_to_index)


def test_jf_mapping():
    spb_jf_mapping = {
        s: i  # special: JF geometry itself subtracts one
        for i, s in enumerate(
            [
                "SPB_IRDA_JF4M/CAL/CORRECT00_M1:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT01_M2:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT02_M3:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT03_M4:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT04_M5:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT05_M6:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT06_M7:preview.outputCorrected",
                "SPB_IRDA_JF4M/CAL/CORRECT07_M8:preview.outputCorrected",
            ],
            start=1,
        )
    }

    d = NotADetectorAssembler()
    subset = list(spb_jf_mapping.keys())
    del subset[2:4]
    d.set("sources", [Hash("select", True, "source", s) for s in subset])
    d.set("sourceToModuleIndex", [])
    d._merge_source_to_index_from_regex()
    d._set_source_to_index_from_table()
    for source in subset:
        assert spb_jf_mapping[source] == d._source_to_index[source]
    assert len(subset) == len(d._source_to_index)
